! =================================================================================================================================
! MODULE 	: constantes_soil
!
! CONTACT       : orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE       : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF         "constantes_soil" module contains public data for the soils.
!!
!!\n DESCRIPTION : The non saturated hydraulic properties are defined from the  
!!                 formulations of van Genuchten (1980) and Mualem (1976), combined as  
!!                 explained in d'Orgeval (2006). \n
!!                 The related parameters for three main  
!!                 soil textures (coarse, medium and fine) come from Carsel and Parrish  
!!                 (1988).
!!
!! RECENT CHANGE(S): Sonke Zaehle changed hcrit_litter value according to Shilong Piao from 0.03 to 0.08, 080806
!!
!! REFERENCE(S)	:
!!- Roger A.Pielke, (2002), Mesoscale meteorological modeling, Academic Press Inc. 
!!- Polcher, J., Laval, K., Dümenil, L., Lean, J., et Rowntree, P. R. (1996).
!! Comparing three land surface schemes used in general circulation models. Journal of Hydrology, 180(1-4), 373--394.
!!- Ducharne, A., Laval, K., et Polcher, J. (1998). Sensitivity of the hydrological cycle
!! to the parametrization of soil hydrology in a GCM. Climate Dynamics, 14, 307--327. 
!!- Rosnay, P. de et Polcher, J. (1999). Modelling root water uptake in a complex land surface
!! scheme coupled to a GCM. Hydrol. Earth Syst. Sci., 2(2/3), 239--255.
!!- d'Orgeval, T. et Polcher, J. (2008). Impacts of precipitation events and land-use changes
!! on West African river discharges during the years 1951--2000. Climate Dynamics, 31(2), 249--262. 
!!- Carsel, R. and Parrish, R.: Developing joint probability distributions of soil water
!! retention characteristics, Water Resour. Res.,24, 755–769, 1988.
!!- Mualem Y (1976) A new model for predicting the hydraulic conductivity  
!! of unsaturated porous media. Water Resources Research 12(3):513-522
!!- Van Genuchten M (1980) A closed-form equation for predicting the  
!! hydraulic conductivity of unsaturated soils. Soil Sci Soc Am J, 44(5):892-898
!!
!! SVN          :
!! $HeadURL: $
!! $Date: $
!! $Revision: $
!! \n
!_ ================================================================================================================================

MODULE constantes_soil

  ! modules used :
  USE constantes
  USE parallel

  IMPLICIT NONE


  LOGICAL, SAVE             :: check_waterbal=.TRUE.    !! The check the water balance (true/false)
!$OMP THREADPRIVATE(check_waterbal)

  !! Dimensioning parameters

  INTEGER(i_std), PARAMETER :: ngrnd=7                  !! Number of soil level (unitless)
  INTEGER(i_std), PARAMETER :: nbdl=11                  !! Number of diagnostic levels in the soil
                                                        !! To compare hydrologic variables with tag 1.6 and lower,
                                                        !! set nbdl to 6 : INTEGER(i_std), PARAMETER :: nbdl = 6
                                                        !! (unitless)
  INTEGER(i_std), PARAMETER :: nslm=11                  !! Number of levels in CWRR (unitless)

  REAL(r_std), SAVE         :: dpu_max = 4.0_r_std      !! Maximum depth of soil reservoir. (m)
                                                        !! This is the default value for Choisnel used for the AR5
                                                        !! simulations. If CWRR is activated, dpu_max is set to 2
                                                        !! in intsurf_config.
!$OMP THREADPRIVATE(dpu_max)

  !! Number of soil classes

  INTEGER(i_std), PARAMETER :: ntext=3                  !! Number of soil textures (Silt, Sand, Clay)
  INTEGER(i_std), PARAMETER :: nstm=3                   !! Number of soil tiles (unitless)
  CHARACTER(LEN=30)         :: soil_classif             !! Type of classification used for the map of soil types.
                                                        !! It must be consistent with soil file given by 
                                                        !! SOILCLASS_FILE parameter.
  INTEGER(i_std), PARAMETER :: nscm_fao=3               !! For FAO Classification (unitless)
  INTEGER(i_std), PARAMETER :: nscm_usda=12             !! For USDA Classification (unitless)
  INTEGER(i_std), SAVE      :: nscm=nscm_fao            !! Default value for nscm
!$OMP THREADPRIVATE(nscm)

  !! Parameters for soil thermodynamics

  REAL(r_std), SAVE :: so_capa_dry = 1.80e+6            !! Dry soil Heat capacity of soils 
                                                        !! @tex $(J.m^{-3}.K^{-1})$ @endtex 
!$OMP THREADPRIVATE(so_capa_dry)
  REAL(r_std), SAVE :: so_cond_dry = 0.40               !! Dry soil Thermal Conductivity of soils
                                                        !! @tex $(W.m^{-2}.K^{-1})$ @endtex
!$OMP THREADPRIVATE(so_cond_dry)
  REAL(r_std), SAVE :: so_capa_wet = 3.03e+6            !! Wet soil Heat capacity of soils 
                                                        !! @tex $(J.m^{-3}.K^{-1})$ @endtex
!$OMP THREADPRIVATE(so_capa_wet)
  REAL(r_std), SAVE :: so_cond_wet = 1.89               !! Wet soil Thermal Conductivity of soils 
                                                        !! @tex $(W.m^{-2}.K^{-1})$ @endtex 
!$OMP THREADPRIVATE(so_cond_wet)
  REAL(r_std), SAVE :: sn_cond = 0.3                    !! Thermal Conductivity of snow 
                                                        !! @tex $(W.m^{-2}.K^{-1})$ @endtex  
!$OMP THREADPRIVATE(sn_cond)
  REAL(r_std), SAVE :: sn_dens = 330.0                  !! Snow density for the soil thermodynamics
                                                        !! (unitless)
!$OMP THREADPRIVATE(sn_dens)
  REAL(r_std), SAVE :: sn_capa                          !! Heat capacity for snow 
                                                        !! @tex $(J.m^{-3}.K^{-1})$ @endtex
!$OMP THREADPRIVATE(sn_capa)


  !! Specific parameters for the Choisnel hydrology

  REAL(r_std), SAVE :: min_drain = 0.001                !! Diffusion constant for the slow regime
                                                        !! (This is for the diffusion between reservoirs)
                                                        !! @tex $(kg.m^{-2}.dt^{-1})$ @endtex
!$OMP THREADPRIVATE(min_drain)
  REAL(r_std), SAVE :: max_drain = 0.1                  !! Diffusion constant for the fast regime 
                                                        !! @tex $(kg.m^{-2}.dt^{-1})$ @endtex
!$OMP THREADPRIVATE(max_drain)
  REAL(r_std), SAVE :: exp_drain = 1.5                  !! The exponential in the diffusion law (unitless)
!$OMP THREADPRIVATE(exp_drain)
  REAL(r_std), SAVE :: qsintcst = 0.1                   !! Transforms leaf area index into size of interception reservoir
                                                        !! (unitless)
!$OMP THREADPRIVATE(qsintcst)
  REAL(r_std), SAVE :: mx_eau_nobio = 150.              !! Volumetric available soil water capacity in nobio fractions
                                                        !! @tex $(kg.m^{-3} of soil)$ @endtex
!$OMP THREADPRIVATE(mx_eau_nobio)
  REAL(r_std), SAVE :: rsol_cste = 33.E3                !! Constant in the computation of resistance for bare soil evaporation
                                                        !! @tex $(s.m^{-2})$ @endtex
!$OMP THREADPRIVATE(rsol_cste)
  REAL(r_std), SAVE :: hcrit_litter=0.08_r_std          !! Scaling depth for litter humidity (m)
!$OMP THREADPRIVATE(hcrit_litter)


  !! Parameters specific for the CWRR hydrology.

  !!  1. Parameters for FAO Classification

  !! Parameters for soil type distribution

  REAL(r_std),DIMENSION(nscm_fao),SAVE :: soilclass_default_fao = &   !! Default soil texture distribution for fao :
 & (/ 0.28, 0.52, 0.20 /)                                             !! in the following order : COARSE, MEDIUM, FINE (unitless)
!$OMP THREADPRIVATE(soilclass_default_fao)

  REAL(r_std),PARAMETER,DIMENSION(nscm_fao) :: nvan_fao = &            !! Van genuchten coefficient n
 & (/ 1.89_r_std, 1.56_r_std, 1.31_r_std /)

  REAL(r_std),PARAMETER,DIMENSION(nscm_fao) :: avan_fao = &            !! Van genuchten coefficient a (mm^{-1}) 
  & (/ 0.0075_r_std, 0.0036_r_std, 0.0019_r_std /) 

  REAL(r_std),PARAMETER,DIMENSION(nscm_fao) :: mcr_fao = &             !! Residual soil water content
 & (/ 0.065_r_std, 0.078_r_std, 0.095_r_std /)

  REAL(r_std),PARAMETER,DIMENSION(nscm_fao) :: mcs_fao = &             !! Saturated soil water content
 & (/ 0.41_r_std, 0.43_r_std, 0.41_r_std /)

  REAL(r_std),PARAMETER,DIMENSION(nscm_fao) :: ks_fao = &              !! Hydraulic conductivity Saturation (mm/d)
 & (/ 1060.8_r_std, 249.6_r_std, 62.4_r_std /)

  REAL(r_std),PARAMETER,DIMENSION(nscm_fao) :: pcent_fao = &           !! Fraction of saturated volumetric soil moisture 
 & (/ 0.5_r_std, 0.5_r_std, 0.5_r_std /)                               !! above which transpir is max (0-1, unitless)

  REAL(r_std),PARAMETER,DIMENSION(nscm_fao) :: free_drain_max_fao = &  !! Max value of the permeability coeff at 
 & (/ 1.0_r_std, 1.0_r_std, 1.0_r_std /)                               !! the bottom of the soil

  REAL(r_std),PARAMETER,DIMENSION(nscm_fao) :: mcf_fao = &             !! Volumetric water content field capacity
 & (/ 0.32_r_std, 0.32_r_std, 0.32_r_std /)

  REAL(r_std),PARAMETER,DIMENSION(nscm_fao) :: mcw_fao = &             !! Volumetric water content Wilting pt
 & (/ 0.10_r_std, 0.10_r_std, 0.10_r_std /)

  REAL(r_std),PARAMETER,DIMENSION(nscm_fao) :: mc_awet_fao = &         !! Vol. wat. cont. above which albedo is cst
 & (/ 0.25_r_std, 0.25_r_std, 0.25_r_std /)

  REAL(r_std),PARAMETER,DIMENSION(nscm_fao) :: mc_adry_fao = &         !! Vol. wat. cont. below which albedo is cst
 & (/ 0.1_r_std, 0.1_r_std, 0.1_r_std /)


  !!  2. Parameters for USDA Classification

  !! Parameters for soil type distribution :
  !! Sand, Loamy Sand, Sandy Loam, Silt Loam, Silt, Loam, Sandy Clay Loam, Silty Clay Loam, Clay Loam, Sandy Clay, Silty Clay, Clay

  REAL(r_std),DIMENSION(nscm_usda),SAVE :: soilclass_default_usda = &    !! Default soil texture distribution in the following order :
 & (/ 0.28, 0.52, 0.20, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 /)   !!    sand, loam and clay ??? OR COARSE, MEDIUM, FINE???
!$OMP THREADPRIVATE(soilclass_default_usda)

  REAL(r_std),PARAMETER,DIMENSION(nscm_usda) :: nvan_usda = &            !! Van genuchten coefficient n
 & (/ 2.68_r_std, 2.28_r_std, 1.89_r_std, 1.41_r_std, &
 &    1.37_r_std, 1.56_r_std, 1.48_r_std, 1.23_r_std, &
 &    1.31_r_std, 1.23_r_std, 1.09_r_std, 1.09_r_std /)

  REAL(r_std),PARAMETER,DIMENSION(nscm_usda) :: avan_usda = &            !! Van genuchten coefficient a (mm^{-1})
 & (/ 0.0145_r_std, 0.0124_r_std, 0.0075_r_std, 0.0020_r_std, &
 &    0.0016_r_std, 0.0036_r_std, 0.0059_r_std, 0.0010_r_std, &
 &    0.0019_r_std, 0.0027_r_std, 0.0005_r_std, 0.0008_r_std /)

  REAL(r_std),PARAMETER,DIMENSION(nscm_usda) :: mcr_usda = &             !! Residual soil water content
 & (/ 0.045_r_std, 0.057_r_std, 0.065_r_std, 0.067_r_std, &
 &    0.034_r_std, 0.078_r_std, 0.100_r_std, 0.089_r_std, &
 &    0.095_r_std, 0.100_r_std, 0.070_r_std, 0.068_r_std /)

  REAL(r_std),PARAMETER,DIMENSION(nscm_usda) :: mcs_usda = &             !! Saturated soil water content
 & (/ 0.43_r_std, 0.41_r_std, 0.41_r_std, 0.45_r_std, &
 &    0.46_r_std, 0.43_r_std, 0.39_r_std, 0.43_r_std, &
 &    0.41_r_std, 0.38_r_std, 0.36_r_std, 0.38_r_std /)

  REAL(r_std),PARAMETER,DIMENSION(nscm_usda) :: ks_usda = &              !! Hydraulic conductivity Saturation (mm/d)
 & (/ 7128.0_r_std, 3501.6_r_std, 1060.8_r_std, 108.0_r_std, &
 &    60.0_r_std, 249.6_r_std, 314.4_r_std, 16.8_r_std, &
 &    62.4_r_std, 28.8_r_std, 4.8_r_std, 48.0_r_std /)

  REAL(r_std),PARAMETER,DIMENSION(nscm_usda) :: pcent_usda = &           !! Soil moisture above which transpir is max
 & (/ 0.5_r_std, 0.5_r_std, 0.5_r_std, 0.5_r_std, &
 &    0.5_r_std, 0.5_r_std, 0.5_r_std, 0.5_r_std, &
 &    0.5_r_std, 0.5_r_std, 0.5_r_std, 0.5_r_std /)

  REAL(r_std),PARAMETER,DIMENSION(nscm_usda) :: free_drain_max_usda = &  !! Max value of the permeability coeff at 
 & (/ 1.0_r_std, 1.0_r_std, 1.0_r_std, 1.0_r_std, &                      !! the bottom of the soil
 &    1.0_r_std, 1.0_r_std, 1.0_r_std, 1.0_r_std, &
 &    1.0_r_std, 1.0_r_std, 1.0_r_std, 1.0_r_std /)

  REAL(r_std),PARAMETER,DIMENSION(nscm_usda) :: mcf_usda = &             !! Volumetric water content field capacity
 & (/ 0.32_r_std, 0.32_r_std, 0.32_r_std, 0.32_r_std, &
 &    0.32_r_std, 0.32_r_std, 0.32_r_std, 0.32_r_std, &
 &    0.32_r_std, 0.32_r_std, 0.32_r_std, 0.32_r_std /)

  REAL(r_std),PARAMETER,DIMENSION(nscm_usda) :: mcw_usda = &             !! Volumetric water content Wilting pt
 & (/ 0.10_r_std, 0.10_r_std, 0.10_r_std, 0.10_r_std, &
 &    0.10_r_std, 0.10_r_std, 0.10_r_std, 0.10_r_std, &
 &    0.10_r_std, 0.10_r_std, 0.10_r_std, 0.10_r_std /)

  REAL(r_std),PARAMETER,DIMENSION(nscm_usda) :: mc_awet_usda = &         !! Vol. wat. cont. above which albedo is cst
 & (/ 0.25_r_std, 0.25_r_std, 0.25_r_std, 0.25_r_std, &
 &    0.25_r_std, 0.25_r_std, 0.25_r_std, 0.25_r_std, &
 &    0.25_r_std, 0.25_r_std, 0.25_r_std, 0.25_r_std /)

  REAL(r_std),PARAMETER,DIMENSION(nscm_usda) :: mc_adry_usda = &         !! Vol. wat. cont. below which albedo is cst
 & (/ 0.1_r_std, 0.1_r_std, 0.1_r_std, 0.1_r_std, &
 &    0.1_r_std, 0.1_r_std, 0.1_r_std, 0.1_r_std, &
 &    0.1_r_std, 0.1_r_std, 0.1_r_std, 0.1_r_std /)


  !! Parameters for the numerical scheme used by CWRR

  INTEGER(i_std), PARAMETER :: imin = 1                                 !! CWRR linearisation (unitless)
  INTEGER(i_std), PARAMETER :: nbint = 50                               !! Number of interval for CWRR (unitless)
  INTEGER(i_std), PARAMETER :: imax = nbint+1                           !! Number of points for CWRR (unitless)
  REAL(r_std), PARAMETER    :: w_time = 1.0_r_std                       !! Time weighting for discretisation (unitless)


  !! Diagnostic variables

  REAL(r_std),DIMENSION(nbdl),SAVE :: diaglev                           !! The lower limit of the layer on which soil moisture
                                                                        !! (relative) and temperature are going to be diagnosed.
                                                                        !! These variables are made for transfering the information
                                                                        !! to the biogeophyical processes modelled in STOMATE. 
                                                                        !! (unitless)
!$OMP THREADPRIVATE(diaglev)

CONTAINS

  
!! ================================================================================================================================
!! SUBROUTINE   : config_soil_parameters
!!
!>\BRIEF        This subroutine reads in the configuration file all the parameters related to soil and hydrology. 
!!
!! DESCRIPTION  : None
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S): 
!!
!! REFERENCE(S) :
!!
!! FLOWCHART    :
!! \n
!_ ================================================================================================================================

   SUBROUTINE config_soil_parameters(active_flags)
     
     USE ioipsl

     IMPLICIT NONE

     !! 0. Variables and parameters declaration

     !! 0.1 Input variables

     TYPE(control_type), INTENT(in) :: active_flags            !! What parts of the code are activated ? (true/false)  

    !! 0.4 Local variables 

     INTEGER(i_std), PARAMETER      :: error_level = 3         !! Switch to 2 to turn fatal errors into warnings.(1-3, unitless)
     LOGICAL, SAVE                  :: first_call = .TRUE.     !! To keep first call trace (true/false)
!$OMP THREADPRIVATE(first_call)

!_ ================================================================================================================================
     
     IF ( first_call ) THEN
        
        !Config Key   = DRY_SOIL_HEAT_CAPACITY
        !Config Desc  = Dry soil Heat capacity of soils
        !Config If    = OK_SECHIBA 
        !Config Def   = 1.80e+6
        !Config Help  = Values taken from : PIELKE,'MESOSCALE METEOROLOGICAL MODELING',P.384.
        !Config Units = [J.m^{-3}.K^{-1}] 
        CALL getin_p("DRY_SOIL_HEAT_CAPACITY",so_capa_dry)
        
        !! Check parameter value (correct range)
        IF ( so_capa_dry <= zero ) THEN
           CALL ipslerr_p(error_level, "config_soil_parameters.", &
                &     "Wrong parameter value for DRY_SOIL_HEAT_CAPACITY.", &
                &     "This parameter should be positive. ", &
                &     "Please, check parameter value in run.def. ")
        END IF
        

        !Config Key   = DRY_SOIL_HEAT_COND
        !Config Desc  = Dry soil Thermal Conductivity of soils
        !Config If    = OK_SECHIBA
        !Config Def   = 0.40 
        !Config Help  = Values taken from : PIELKE,'MESOSCALE METEOROLOGICAL MODELING',P.384.
        !Config Units = [W.m^{-2}.K^{-1}] 
        CALL getin_p("DRY_SOIL_HEAT_COND",so_cond_dry)

        !! Check parameter value (correct range)
        IF ( so_cond_dry <= zero ) THEN
           CALL ipslerr_p(error_level, "config_soil_parameters.", &
                &     "Wrong parameter value for DRY_SOIL_HEAT_COND.", &
                &     "This parameter should be positive. ", &
                &     "Please, check parameter value in run.def. ")
        END IF


        !Config Key   = WET_SOIL_HEAT_CAPACITY
        !Config Desc  = Wet soil Heat capacity of soils 
        !Config If    = OK_SECHIBA
        !Config Def   = 3.03e+6
        !Config Help  = 
        !Config Units = [J.m^{-3}.K^{-1}]
        CALL getin_p("WET_SOIL_HEAT_CAPACITY",so_capa_wet)

        !! Check parameter value (correct range)
        IF ( so_capa_wet <= zero ) THEN
           CALL ipslerr_p(error_level, "config_soil_parameters.", &
               &     "Wrong parameter value for WET_SOIL_HEAT_CAPACITY.", &
               &     "This parameter should be positive. ", &
               &     "Please, check parameter value in run.def. ")
        END IF


        !Config Key   = WET_SOIL_HEAT_COND
        !Config Desc  = Wet soil Thermal Conductivity of soils
        !Config If    = OK_SECHIBA 
        !Config Def   = 1.89 
        !Config Help  = 
        !Config Units = [W.m^{-2}.K^{-1}]
        CALL getin_p("WET_SOIL_HEAT_COND",so_cond_wet)

        !! Check parameter value (correct range)
        IF ( so_cond_wet <= zero ) THEN
           CALL ipslerr_p(error_level, "config_soil_parameters.", &
               &     "Wrong parameter value for WET_SOIL_HEAT_COND.", &
               &     "This parameter should be positive. ", &
               &     "Please, check parameter value in run.def. ")
        END IF


        !Config Key   = SNOW_HEAT_COND
        !Config Desc  = Thermal Conductivity of snow
        !Config If    = OK_SECHIBA  
        !Config Def   = 0.3
        !Config Help  = 
        !Config Units = [W.m^{-2}.K^{-1}]
        CALL getin_p("SNOW_HEAT_COND",sn_cond)

        !! Check
        IF ( sn_cond <= zero ) THEN
           CALL ipslerr_p(error_level, "config_soil_parameters.", &
               &     "Wrong parameter value for SNOW_HEAT_COND.", &
               &     "This parameter should be positive. ", &
               &     "Please, check parameter value in run.def. ")
        END IF


        !Config Key   = SNOW_DENSITY
        !Config Desc  = Snow density for the soil thermodynamics 
        !Config If    = OK_SECHIBA 
        !Config Def   = 330.0
        !Config Help  = 
        !Config Units = [-] 
        CALL getin_p("SNOW_DENSITY",sn_dens)
        
        !! Check parameter value (correct range)
        IF ( sn_dens <= zero ) THEN
          CALL ipslerr_p(error_level, "config_soil_parameters.", &
               &     "Wrong parameter value for SNOW_DENSITY.", &
               &     "This parameter should be positive. ", &
               &     "Please, check parameter value in run.def. ")
        END IF


        !! Calculation of snow capacity
        !! If sn_dens is redefined by the user, sn_capa needs to be reset
        sn_capa = 2100.0_r_std*sn_dens


        !Config Key   = NOBIO_WATER_CAPAC_VOLUMETRI
        !Config Desc  = 
        !Config If    = OK_SECHIBA and .NOT.(HYDROL_CWRR)
        !Config Def   = 150.
        !Config Help  = 
        !Config Units = [s/m^2]
        CALL getin_p('NOBIO_WATER_CAPAC_VOLUMETRI',mx_eau_nobio)

       !! Check parameter value (correct range)
        IF ( mx_eau_nobio <= zero ) THEN
           CALL ipslerr_p(error_level, "config_soil_parameters.", &
               &     "Wrong parameter value for NOBIO_WATER_CAPAC_VOLUMETRI.", &
               &     "This parameter should be positive. ", &
               &     "Please, check parameter value in run.def. ")
        END IF


        !Config Key   = SECHIBA_QSINT 
        !Config Desc  = Interception reservoir coefficient
        !Config If    = OK_SECHIBA 
        !Config Def   = 0.1
        !Config Help  = Transforms leaf area index into size of interception reservoir
        !Config         for slowproc_derivvar or stomate
        !Config Units = [m]
        CALL getin_p('SECHIBA_QSINT',qsintcst)

        !! Check parameter value (correct range)
        IF ( qsintcst <= zero ) THEN
           CALL ipslerr_p(error_level, "config_soil_parameters.", &
                &     "Wrong parameter value for SECHIBA_QSINT.", &
                &     "This parameter should be positive. ", &
                &     "Please, check parameter value in run.def. ")
        END IF


        IF ( .NOT.(active_flags%hydrol_cwrr) ) THEN
           
           !Config Key   = CHOISNEL_DIFF_MIN
           !Config Desc  = Diffusion constant for the slow regime
           !Config If    = OK_SECHIBA and .NOT.(HYDROL_CWRR)
           !Config Def   = 0.001
           !Config Help  = 
           !Config Units = [kg/m^2/dt]
           CALL getin_p('CHOISNEL_DIFF_MIN',min_drain)

           !! Check parameter value (correct range)
           IF ( min_drain <= zero ) THEN
              CALL ipslerr_p(error_level, "config_soil_parameters.", &
                   &     "Wrong parameter value for CHOISNEL_DIFF_MIN.", &
                   &     "This parameter should be positive. ", &
                   &     "Please, check parameter value in run.def. ")
            END IF


           !Config Key   = CHOISNEL_DIFF_MAX
           !Config Desc  = Diffusion constant for the fast regime
           !Config If    = OK_SECHIBA and .NOT.(HYDROL_CWRR)
           !Config Def   = 0.1
           !Config Help  = 
           !Config Units = [kg/m^2/dt]
           CALL getin_p('CHOISNEL_DIFF_MAX',max_drain)

           !! Check parameter value (correct range)
           IF (  ( max_drain <= zero ) .OR. ( max_drain <= min_drain ) ) THEN
              CALL ipslerr_p(error_level, "config_soil_parameters.", &
                   &     "Wrong parameter value for CHOISNEL_DIFF_MAX.", &
                   &     "This parameter should be positive or greater than CHOISNEL_DIFF_MIN.", &
                   &     "Please, check parameter value in run.def. ")
           END IF


           !Config Key   = CHOISNEL_DIFF_EXP
           !Config Desc  = The exponential in the diffusion law
           !Config If    = OK_SECHIBA and .NOT.(HYDROL_CWRR)
           !Config Def   = 1.5
           !Config Help  = 
           !Config Units = [-]
           CALL getin_p('CHOISNEL_DIFF_EXP',exp_drain)
           
           !! Check parameter value (correct range)
           IF ( exp_drain <= zero ) THEN
              CALL ipslerr_p(error_level, "config_soil_parameters.", &
                   &     "Wrong parameter value for CHOISNEL_DIFF_EXP.", &
                   &     "This parameter should be positive. ", &
                   &     "Please, check parameter value in run.def. ")
           END IF


           !Config Key   = CHOISNEL_RSOL_CSTE
           !Config Desc  = Constant in the computation of resistance for bare  soil evaporation 
           !Config If    = OK_SECHIBA and .NOT.(HYDROL_CWRR)
           !Config Def   = 33.E3
           !Config Help  = 
           !Config Units = [s/m^2]
           CALL getin_p('CHOISNEL_RSOL_CSTE',rsol_cste)

           !! Check parameter value (correct range)
           IF ( rsol_cste <= zero ) THEN
              CALL ipslerr_p(error_level, "config_soil_parameters.", &
                   &     "Wrong parameter value for CHOISNEL_RSOL_CSTE.", &
                   &     "This parameter should be positive. ", &
                   &     "Please, check parameter value in run.def. ")
           END IF


           !Config Key   = HCRIT_LITTER
           !Config Desc  = Scaling depth for litter humidity
           !Config If    = OK_SECHIBA and .NOT.(HYDROL_CWRR) 
           !Config Def   = 0.08 
           !Config Help  = 
           !Config Units = [m]
           CALL getin_p('HCRIT_LITTER',hcrit_litter)

           !! Check parameter value (correct range)
           IF ( hcrit_litter <= zero ) THEN
              CALL ipslerr_p(error_level, "config_soil_parameters.", &
                   &     "Wrong parameter value for HCRIT_LITTER.", &
                   &     "This parameter should be positive. ", &
                   &     "Please, check parameter value in run.def. ")
           END IF


        END IF

        first_call =.FALSE.
        
     ENDIF
     
   END SUBROUTINE config_soil_parameters
   

END MODULE constantes_soil
